/*	LazyBridge V1.2 by Hrvoje Cop aka chopmeister / Studio Niskota - www.studio-niskota.hr

   Usage: when you are bridging polygons on a wall, for example to make a hole for a window,
   you have to select polygons on both sides of the wall. This script will select the opposite polygons 
   (meaning the closest face along the current face normal, depending on chosen direction) for you, 
   so you have to spend less than half the time doing selections. It also works if you are connecting two wall segments 
   (for example to make a beam). You must only choose which way the script will look for matching polygons: inward (windows) or 
  outward (beams). 	

  */

/* CHANGELOG:

		V1.2 - fixed the problem of the script not working if the selected editable poly has modifiers
				  - removed the GO button - now the script automatically bridges the polygons after selecting

*/
 



rollout LazyBridge "L.B."  width: 95 height: 52	--declare rollout elements
(
	 	
	button btnInward "" pos:[6,6]  width: 40 height: 40 tooltip: "Search along inverted face normal."  images: #("$scripts\ChopmeisterScripts\images\LazyBridge_buttons.bmp","$scripts\ChopmeisterScripts\images\LazyBridge_Mask.bmp",3,1,1,1,1)
	button btnOutward "" pos:[48,6]  width: 40 height: 40 tooltip: "Search along face normal."  images: #("$scripts\ChopmeisterScripts\images\LazyBridge_buttons.bmp","$scripts\ChopmeisterScripts\images\LazyBridge_Mask.bmp",3,2,2,2,2)
	
	
	/* the main function that finds matches to selected faces by performing rayMeshGridIntersect() checks
	   on a copied object */
	
	fn selectMatchingFaces sceneObject dSwitch =
	(
			
					
			
			newSelection = #{}
			
			oldPolyFaceSelection = polyop.getFaceSelection sceneObject --get original face selection before going to max create mode
			
			max create mode  --go to create mode for extra speed
			
			theObject = copy sceneObject	--make a copy of the scene object,  delete modifiers from it, and convert to mesh to get its face selection							
			
			for m in theObject.modifiers do deleteModifier theObject m 
			
			convertToMesh theObject
			
			
			theSelection = getFaceSelection theObject
			

			rm = rayMeshGridIntersect ()	--Initialize and build the rayMeshGridIntersect() thingamajig
			rm.Initialize 20
			rm.addNode theObject
			rm.buildGrid ()		
			
			for i in theSelection do 	--go through all the selected faces and find their matches depending on the direction chosen
			(												
					currentFaceNorm = in coordsys world (getFaceNormal theObject i)
					currentFaceCenter = in coordsys world (meshop.getFaceCenter theObject i)							
			
					/*now shoot the ray according to the directionSwitch parameter. if the chosen direction is "inward" move the starting point of the ray a tiny bit inward to 
					  prevent hitting the already selected face*/ 
			
					if dSwitch == true then hitCount = rm.intersectRay (currentFaceCenter-(currentFaceNorm*0.01))  -(currentFaceNorm) true	--if inward look for  flipped faces					
					else if dSwitch == false then hitCount = rm.intersectRay currentFaceCenter currentFaceNorm false
					
					
					if hitCount > 0 do 	--if anything was hit get the index of the closest face and append it to the new selection
					(
							closestFaceIndex = rm.getClosestHit()
							closestFace = (rm.getHitFace closestFaceIndex)
							append newSelection closestFace
					)
			)	
		
			rm.free()	--free the memory from rayMeshGridIntersect()											
			
			setFaceSelection theObject newSelection	--set the new selection
			convertToPoly theObject	--convert the mesh copy to an editable poly to get the actual poly face selection
			
			newPolyFaceSelection = polyop.getFaceSelection theObject       --get the new faces from the copied object and old faces from the original
				
			max modify mode	--return to where we started - select the object and go to polygon subobject level of the modify panel
			modPanel.setCurrentObject $.baseObject
			subobjectlevel=4
				
			/* now we combine the selections. it needs to be done like this:
			-select one new face and get its vertices
			-use those vertices to find to corresponding face on the original object
			-add that face to the array holding the original selection
			
			otherwise if you chose two polygons exactly one poly apart, you would get all of the vertices of that middle poly in the vertex list
			and it would also get selected */
				
			for i in newPolyFaceSelection do 
			(
					fooVerts=polyop.getVertsUsingFace theObject i
					polyop.setVertSelection sceneObject fooVerts
					sceneObject.convertSelection 1 4 requireAll:true
					fooFace=polyop.getFaceSelection sceneObject
					oldPolyFaceSelection += fooFace
			)
			
			delete theObject 	--delete the copied object

			polyop.setFaceSelection sceneObject oldPolyFaceSelection  --finally select the final set of faces
			
			    
			
	)
	
	on btnInward pressed do 
	(
			clearListener()
			
			undo "LazyBridge" on
			(
				viewportSelectedObj= selection[1]	
				
				if  classOf viewportSelectedObj != Editable_Poly  then messagebox "Please select an editable poly!" title:"LazyBridge Error!"      --throw error if not editable_poly or if not in appropriate subobject level
				else if subobjectlevel != 4 then messagebox "Please select faces in an editable poly!" title:"LazyBridge Error!"
				
				else 
				(			 			 
						start = timeStamp()
						clearListener()
						with redraw off (	--disable redraw for extra speed
						selectMatchingFaces viewportSelectedObj true
						viewportSelectedObj.bridge()
						end = timeStamp()
						format "LazyBridge took % seconds\n" ((end - start) / 1000.0)		
						)		
				)			
			)	
	)		

on btnOutward pressed do 
	(
			clearListener()
			
			undo "LazyBridge" on
			(
				viewportSelectedObj= selection[1]	
				
				if  classOf viewportSelectedObj != Editable_Poly  then messagebox "Please select an editable poly!" title:"LazyBridge Error!"      --throw error if not editable_poly or if not in appropriate subobject level
				else if subobjectlevel != 4 then messagebox "Please select faces in an editable poly!" title:"LazyBridge Error!"
				
				else 
				(			 			 
						start = timeStamp()
						clearListener()
						with redraw off (	--disable redraw for extra speed
						selectMatchingFaces viewportSelectedObj false
						viewportSelectedObj.bridge()
						end = timeStamp()
						format "LazyBridge took % seconds\n" ((end - start) / 1000.0)		
						)		
				)			
			)	
	)		
	
)	
